'From Smalltalk/X, Version:2.10.5 on 30-apr-1995 at 8:48:16 am'!

View subclass:#ProgressIndicator
	 instanceVariableNames:'percentage showPercentage fgColor'
	 classVariableNames:''
	 poolDictionaries:''
	 category:'Views-Misc'
!

!ProgressIndicator class methodsFor:'documentation'!

version
"
$Header: /cvs/stx/stx/libwidg2/ProgressIndicator.st,v 1.1 1995-05-03 01:11:23 claus Exp $
"
!

documentation
"
    a view showing a rectangle filled according the percentage value.
    Can be used as a progress indicator a la MSwindows.
"
!

examples
"
    typical progress indicator:

      |top p h|

      top := StandardSystemView new.
      top extent:300@100.
      top label:'Progress'.
      p := ProgressIndicator in:top.
      p origin:(0.0@0.5) corner:(1.0@0.5).
      p level:-1.
      h := p preferedExtent y.
      p topInset:(h // 2) negated; 
	bottomInset:(h // 2) negated;
	leftInset:5;
	rightInset:5.
      top open.

      [
	  1 to:100 do:[:val |
	      (Delay forSeconds:0.05) wait.
	      p percentage:val 
	  ]
      ] fork


    changing colors, turning percentage display off:

      |top p h|

      top := StandardSystemView new.
      top extent:300@100.
      top label:'Progress'.
      p := ProgressIndicator in:top.
      p origin:(0.0@0.5) corner:(1.0@0.5).
      p level:-1.
      p showPercentage:false.
      p foregroundColor:(Color red).
      h := 10.
      p topInset:(h // 2) negated; 
	bottomInset:(h // 2) negated;
	leftInset:5;
	rightInset:5.
      top open.
      [
	  1 to:100 do:[:val |
	      (Delay forSeconds:0.05) wait.
	      p percentage:val 
	  ]
      ] fork

    with border (2D look):

      |top p h|

      top := StandardSystemView new.
      top extent:300@100.
      top label:'Progress'.
      p := ProgressIndicator in:top.
      p origin:(0.0@0.5) corner:(1.0@0.5).
      p borderWidth:1.
      h := p preferedExtent y.
      p topInset:(h // 2) negated; 
	bottomInset:(h // 2) negated;
	leftInset:5;
	rightInset:5.
      top open.
      [
	  1 to:100 do:[:val |
	      (Delay forSeconds:0.05) wait.
	      p percentage:val 
	  ]
      ] fork


    getting progress from a model:

      |model top p h|

      model := 0 asValue.

      top := StandardSystemView new.
      top extent:300@100.
      top label:'Progress'.
      p := ProgressIndicator in:top.
      p model:model.
      p origin:(0.0@0.5) corner:(1.0@0.5).
      p level:-1.
      h := p preferedExtent y.
      p topInset:(h // 2) negated; 
	bottomInset:(h // 2) negated;
	leftInset:5;
	rightInset:5.
      top open.

      [
	  1 to:100 do:[:val |
	      (Delay forSeconds:0.05) wait.
	      model value:val 
	  ]
      ] fork

    concrete example:
      search all files in the source directory for a string
      using grep. Show progress while doing so.

      |top p h names done|

      top := StandardSystemView new.
      top extent:300@100.
      top label:'Searching ...'.
      p := ProgressIndicator in:top.
      p origin:(0.0@0.5) corner:(1.0@0.5).
      p level:-1.
      h := p preferedExtent y.
      p topInset:(h // 2) negated; 
	bottomInset:(h // 2) negated;
	leftInset:5;
	rightInset:5.
      top openWithPriority:(Processor activePriority + 1).

      names := 'source' asFilename directoryContents.
      done := 0.
      names do:[:aName |
	|stream line|

	p percentage:(done / names size * 100).
	stream := ('source/' , aName) asFilename readStream.
	[stream atEnd] whileFalse:[
	    line := stream nextLine.
	    (line findString:'subclass:') ~~ 0 ifTrue:[
		Transcript showCr:line
	    ].
	].
	stream close.
	done := done + 1
      ].

      top destroy
"
! !

!ProgressIndicator methodsFor:'drawing'!

redraw
    |s rx sx sy sw m w h|

    m := margin + 1.
    w := width - (m*2).
    h := height - (m*2).

    s := percentage printString , ' %'.
    sw := font widthOf:s.
    sx := (width - sw) // 2.
    sy := height // 2 + font descent + 2.

    rx := (w * percentage / 100) rounded.

    self paint:Color white.
    self fillRectangleX:m y:m width:w height:h.

    showPercentage ifTrue:[
	rx <= (sx+sw) ifTrue:[
	    self paint:Color black.
	    self displayString:s x:sx y:sy.
	]
    ].

    self paint:fgColor.
    self fillRectangleX:m y:m width:rx height:h.

    showPercentage ifTrue:[
	rx >= sx ifTrue:[
	    self clipRect:(m@m corner:rx+1 @ h).
	    self paint:Color white.
	    self displayString:s x:sx y:sy.
	    self clipRect:nil
	]
    ]
! !

!ProgressIndicator methodsFor:'initialization'!

initialize
    super initialize.
    viewBackground := StyleSheet colorAt:'progressIndicatorViewBackground' default:Color white.
    fgColor := StyleSheet colorAt:'progressIndicatorForegroundColor' default:Color blue.
    percentage := 0.
    showPercentage := true.
! !

!ProgressIndicator methodsFor:'accessing'!

percentage:aNumber
    |newPercentage|

    newPercentage := ((aNumber max:0) min:100) rounded.
    newPercentage ~~ percentage ifTrue:[
	percentage := newPercentage.
	shown ifTrue:[self redraw].
    ]
!

showPercentage:aBoolean
    showPercentage := aBoolean
!

foregroundColor:aColor 
    fgColor := aColor
! !

!ProgressIndicator methodsFor:'queries'!

preferedExtent
    ^ 100 @ (font height + font descent + ((margin + 1) * 2))
! !

!ProgressIndicator methodsFor:'change & update'!

update:aspect with:aParameter from:changedObject
    (aspect == aspectMsg
    and:[changedObject == model]) ifTrue:[
	self percentage:(model perform:aspectMsg).
	^ self
    ].
    ^ super update:aspect with:aParameter from:changedObject
! !

